from invoke import task
import getpass
from lcconfig import (
    ConsoleSettingsConfig,
    DynamicDNSConfig,
    SyncPublisherConfig,
    SymmetricKeysConfig,
)
from console_base.console_keyring import ConsoleKeyring
from uuid import UUID
from .cache import flush
from ..utils import odoo_impersonation_data
from ..settings import CONF_DIR


@task(
    help={
        'cid': (
            'Canonical ID of the failed Appliance that will be impersonated by this Appliance. '
            'The Odoo record must define the "Device DNS Target" of the desired spare Appliance.'
        ),
    },
    pre=[flush],
)
def impersonate(ctx, cid):  # noqa
    """
    This appliance will impersonate another filter console on a temporary basis.

    This appliance cache and database will be reset, and a backup from the failed
    appliance restored on this system. Do not use this command unless you understand
    the implications!
    """
    try:
        UUID(cid)
    except (AttributeError, ValueError, TypeError):
        raise SystemExit(f'\n{cid!r} is not a valid Canonical ID. Please try again.\n')

    try:
        otp = getpass.getpass('Impersonation code from Odoo: ')
    except KeyboardInterrupt:
        return

    print(f'\n# ---------------------------------------------------------------')
    print('Retrieving Appliance data from Odoo\n')
    data = odoo_impersonation_data(cid, otp)

    # Extract these fields from the Odoo-provided data and save the values
    # to the designated locations.
    config_fields = (
        'adminurl',
        'auth_token',
        'cid',
        'device_dns_name',
        'name',
        'rebranded',
        'theme',
    )

    # The impersonated system's data will get saved to the default location,
    # and the impersonator data to another location, so that we can tell the
    # system is impersonating another system rather than being a self-standing
    # unit. But the impersonator should still be reachable via its own DNS names
    # and the VPN settings and Redwood CAs should be issued to the impersonator.
    impersonated_config_data = {}
    impersonated_data = data.get('impersonated', {})
    impersonated_name = impersonated_data['name']
    for field in config_fields:
        impersonated_config_data[field] = impersonated_data[field]

    print(f'Saving Console settings for {impersonated_name}')
    cfg = ConsoleSettingsConfig()
    cfg.save_section('BoxData', impersonated_config_data)

    print(f'Saving Dynamic DNS settings for {impersonated_name}')
    cfg = DynamicDNSConfig()
    cfg.save_section(cfg.main_section, {'url': impersonated_data['ddns_url']})

    print(f'Saving Sync Publisher settings for {impersonated_name}')
    cfg = SyncPublisherConfig()
    cfg.save_section(cfg.main_section, impersonated_data['sync_publisher_data'])

    impersonator_config_data = {}
    impersonator_data = data.get('impersonator', {})

    print(f'Saving Console settings for Impersonator {impersonator_data["name"]}')
    for field in config_fields:
        impersonator_config_data[field] = impersonator_data[field]

    config_file = f'{CONF_DIR}/impersonator.conf'
    cfg = ConsoleSettingsConfig(config_settings=config_file, persist_file=config_file)
    cfg.save_section('BoxData', impersonator_config_data)

    cfg = SymmetricKeysConfig()
    fernet_key = impersonated_data['fernet']
    cfg.save_section(data={'chacha': impersonated_data['chacha'], 'fernet': fernet_key})
    kr = ConsoleKeyring()
    kr.set_password('console', 'fernet', fernet_key)

    print(f'\n# ---------------------------------------------------------------')
    print(f'Initial configuration settings complete')
    print(f'You can now restore {impersonated_name!r} database via Capsule\n')


__all__ = [
    'impersonate',
]
