from invoke import task

from ..utils.health import (
    check_file_ownership,
    check_process,
    check_pgbouncer,
    check_postgres,
    check_disk,
    check_http_reachable,
    check_port_reachable,
)
from ..utils.osutils import get_database_service_name


@task
def health(ctx):
    """
    Check overall system health of Log Cabin Console processes
    """
    for func in (
            check_file_ownership,
            check_disk,
            check_postgres,
            check_pgbouncer,
    ):
        print()
        func()

    for process in ('nginx', 'postmaster', 'redis-server'):
        check_process(process)

    check_port_reachable(port=1517, name='wsgi server')
    check_port_reachable(port=10009, name='ivylink')

    check_http_reachable()


@task
def restart(ctx):
    """Restart the Log Cabin Console and Scheduler"""
    ctx.run('service supervisord restart')


@task(post=[restart])
def nuclear_restart(ctx):
    """
    Force restart all processes associated with the Log Cabin console

    Do not run unless 'health' indicates memory usage is high,
    or a simple 'restart' didn't restore console functionality.
    """
    processes = ['nginx', 'redis', 'pgbouncer', 'supervisord']
    db_service_name = get_database_service_name()
    if db_service_name:
        processes.insert(0, db_service_name)

    for process in processes:
        ctx.run(f'service {process} restart')


@task
def restart_logtailer(ctx):
    """Restart the Real Time Log Viewer stream"""
    ctx.run('supervisorctl restart logtailer')


@task
def restart_webserver(ctx):
    """Restart the Console Web Server"""
    ctx.run('service nginx restart')


__all__ = (
    'health',
    'restart',
    'nuclear_restart',
    'restart_logtailer',
    'restart_webserver',
)
